"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var React = require("react");
var react_router_dom_1 = require("react-router-dom");
var react_scroll_1 = require("react-scroll");
var validUrl = require("valid-url");
var types_1 = require("../types");
var constants_1 = require("../utils/constants");
/**
 * A generic link component which let's the developer render internal, external and scroll-to-hash links, and
 * their associated behaviors with a single link component. Many times we want a menu including a combination of
 * internal, external and scroll links and the abstraction of the differences of rendering each types of link
 * makes it much easier to do so.
 */
var Link = /** @class */ (function (_super) {
    __extends(Link, _super);
    function Link(props) {
        var _this = _super.call(this, props) || this;
        _this._outerReactScrollSpan = null;
        return _this;
    }
    Link.prototype.render = function () {
        var _this = this;
        var type;
        var isReactRoute = _.startsWith(this.props.to, '/');
        var isExternal = validUrl.isWebUri(this.props.to) || _.startsWith(this.props.to, 'mailto:');
        if (isReactRoute) {
            type = types_1.LinkType.ReactRoute;
        }
        else if (isExternal) {
            type = types_1.LinkType.External;
        }
        else {
            type = types_1.LinkType.ReactScroll;
        }
        if (type === types_1.LinkType.ReactScroll && this.props.shouldOpenInNewTab) {
            throw new Error("Cannot open LinkType.ReactScroll links in new tab. link.to: " + this.props.to);
        }
        var styleWithDefault = {
            textDecoration: this.props.textDecoration,
            cursor: 'pointer',
            color: this.props.fontColor,
        };
        switch (type) {
            case types_1.LinkType.External:
                return (React.createElement("a", { target: this.props.shouldOpenInNewTab ? '_blank' : '', className: this.props.className, style: styleWithDefault, href: this.props.to, onMouseOver: this.props.onMouseOver, onMouseEnter: this.props.onMouseEnter, onMouseLeave: this.props.onMouseLeave }, this.props.children));
            case types_1.LinkType.ReactRoute:
                return (React.createElement(react_router_dom_1.NavLink, { to: this.props.to, className: this.props.className, style: styleWithDefault, target: this.props.shouldOpenInNewTab ? '_blank' : '', onMouseOver: this.props.onMouseOver, onMouseEnter: this.props.onMouseEnter, onMouseLeave: this.props.onMouseLeave, activeStyle: this.props.activeStyle }, this.props.children));
            case types_1.LinkType.ReactScroll:
                return (React.createElement("span", { ref: function (input) { return (_this._outerReactScrollSpan = input); }, onMouseOver: this.props.onMouseOver, onMouseEnter: this.props.onMouseEnter, onMouseLeave: this.props.onMouseLeave },
                    React.createElement(react_scroll_1.Link, { to: this.props.to, offset: 0, spy: true, hashSpy: true, duration: constants_1.constants.DOCS_SCROLL_DURATION_MS, containerId: constants_1.constants.SCROLL_CONTAINER_ID, className: this.props.className, style: styleWithDefault, onSetActive: this._onActivityChanged.bind(this, true), onSetInactive: this._onActivityChanged.bind(this, false) },
                        React.createElement("span", { onClick: this._onClickPropagateClickEventAroundScrollLink.bind(this) }, this.props.children))));
            default:
                throw new Error("Unrecognized LinkType: " + type);
        }
    };
    Link.prototype._onActivityChanged = function (isActive) {
        if (this.props.onActivityChanged) {
            this.props.onActivityChanged(isActive);
        }
    };
    // HACK(fabio): For some reason, the react-scroll link decided to stop the propagation of click events.
    // We do however rely on these events being propagated in certain scenarios (e.g when the link
    // is within a dropdown we want to close upon being clicked). Because of this, we register the
    // click event of an inner span, and pass it around the react-scroll link to an outer span.
    Link.prototype._onClickPropagateClickEventAroundScrollLink = function () {
        if (this._outerReactScrollSpan !== null) {
            this._outerReactScrollSpan.click();
        }
    };
    Link.defaultProps = {
        shouldOpenInNewTab: false,
        className: '',
        onMouseOver: _.noop.bind(_),
        onMouseLeave: _.noop.bind(_),
        onMouseEnter: _.noop.bind(_),
        textDecoration: 'none',
        fontColor: 'inherit',
    };
    return Link;
}(React.Component));
exports.Link = Link;
//# sourceMappingURL=link.js.map